@@ -299,6 +299,7 @@ struct proxy_callback_data {
 static void proxy_callback(struct l_dbus_message *message, void *user_data)
 {
 	struct proxy_callback_data *callback_data = user_data;
+	const struct proxy_interface *proxy;
 	const char *name;
 	const char *text;
 
@@ -308,9 +309,16 @@ static void proxy_callback(struct l_dbus_message *message, void *user_data)
 	if (command_is_interactive_mode())
 		return;
 
-	if (l_dbus_message_get_error(message, &name, &text))
+	if (l_dbus_message_get_error(message, &name, &text)) {
 		command_set_exit_status(EXIT_FAILURE);
+		goto quit;
+	}
 
+	proxy = callback_data->user_data;
+	if (!strcmp(proxy->type->interface, IWD_AGENT_MANAGER_INTERFACE))
+		return;
+
+quit:
 	l_main_quit();
 }
 
@@ -643,7 +651,9 @@ static void interfaces_added_callback(struct l_dbus_message *message,
 	if (dbus_message_has_error(message))
 		return;
 
-	l_dbus_message_get_arguments(message, "oa{sa{sv}}", &path, &object);
+	if (!l_dbus_message_get_arguments(message, "oa{sa{sv}}", &path,
+								&object))
+		return;
 
 	proxy_interface_create(path, &object);
 }
@@ -659,7 +669,8 @@ static void interfaces_removed_callback(struct l_dbus_message *message,
 	if (dbus_message_has_error(message))
 		return;
 
-	l_dbus_message_get_arguments(message, "oas", &path, &interfaces);
+	if (!l_dbus_message_get_arguments(message, "oas", &path, &interfaces))
+		return;
 
 	while (l_dbus_message_iter_next_entry(&interfaces, &interface)) {
 		proxy = proxy_interface_find(interface, path);
@@ -691,31 +702,55 @@ static void get_managed_objects_callback(struct l_dbus_message *message,
 		return;
 	}
 
-	l_dbus_message_get_arguments(message, "a{oa{sa{sv}}}", &objects);
+	if (!l_dbus_message_get_arguments(message, "a{oa{sa{sv}}}", &objects)) {
+		l_error("Failed to parse IWD dbus objects, quitting...\n");
 
-	while (l_dbus_message_iter_next_entry(&objects, &path, &object))
-		proxy_interface_create(path, &object);
+		if (!command_is_interactive_mode())
+			command_set_exit_status(EXIT_FAILURE);
 
-	if (!command_is_interactive_mode()) {
-		command_noninteractive_trigger();
+		l_main_quit();
 
 		return;
 	}
 
+	while (l_dbus_message_iter_next_entry(&objects, &path, &object))
+		proxy_interface_create(path, &object);
+
+	if (command_needs_no_agent())
+		goto no_agent;
+
 	if (!agent_manager_register_agent()) {
+		display_error("Failed to register Agent.\n");
+
+		if (!command_is_interactive_mode())
+			command_set_exit_status(EXIT_FAILURE);
+
 		l_main_quit();
 
 		return;
 	}
 
+no_agent:
+	if (!command_is_interactive_mode()) {
+		command_noninteractive_trigger();
+
+		return;
+	}
+
 	display_enable_cmd_prompt();
 }
 
+static void get_managed_objects(void)
+{
+	l_dbus_method_call(dbus, IWD_SERVICE, IWD_ROOT_PATH,
+					L_DBUS_INTERFACE_OBJECT_MANAGER,
+					"GetManagedObjects", NULL,
+					get_managed_objects_callback,
+					NULL, NULL);
+}
+
 static void service_appeared_callback(struct l_dbus *dbus, void *user_data)
 {
-	if (!command_is_interactive_mode())
-		goto get_objects;
-
 	l_dbus_add_signal_watch(dbus, IWD_SERVICE, IWD_ROOT_PATH,
 					L_DBUS_INTERFACE_OBJECT_MANAGER,
 					"InterfacesAdded", L_DBUS_MATCH_NONE,
@@ -730,12 +765,8 @@ static void service_appeared_callback(struct l_dbus *dbus, void *user_data)
 					L_DBUS_INTERFACE_PROPERTIES,
 					"PropertiesChanged", L_DBUS_MATCH_NONE,
 					properties_changed_callback, NULL);
-get_objects:
-	l_dbus_method_call(dbus, IWD_SERVICE, IWD_ROOT_PATH,
-					L_DBUS_INTERFACE_OBJECT_MANAGER,
-					"GetManagedObjects", NULL,
-					get_managed_objects_callback,
-					NULL, NULL);
+
+	get_managed_objects();
 }
 
 static void service_disappeared_callback(struct l_dbus *dbus,
@@ -805,9 +836,13 @@ bool dbus_proxy_init(void)
 	l_dbus_set_disconnect_handler(dbus, dbus_disconnect_callback, NULL,
 									NULL);
 
-	l_dbus_add_service_watch(dbus, IWD_SERVICE, service_appeared_callback,
+	if (command_is_interactive_mode())
+		l_dbus_add_service_watch(dbus, IWD_SERVICE,
+						service_appeared_callback,
 						service_disappeared_callback,
 						NULL, NULL);
+	else
+		get_managed_objects();
 
 	return true;
 }
@@ -816,7 +851,7 @@ bool dbus_proxy_exit(void)
 {
 	struct interface_type_desc *desc;
 
-	if (command_is_interactive_mode())
+	if (!command_needs_no_agent())
 		agent_manager_unregister_agent();
 
 	for (desc = __start___interface; desc < __stop___interface; desc++) {
